@@ -4,7 +4,7 @@ package urpm::select;
 
 use strict;
 use urpm::msg;
-use urpm::util;
+use urpm::util qw(any formatList intersection member min partition uniq);
 use urpm::sys;
 use URPM;
 
@@ -94,6 +94,8 @@ sub build_listid_ {
 
 Search packages registered by their names by storing their ids into the $packages hash.
 
+Returns either 0 (error), 1 (OK) or 'substring' (fuzzy match).
+
 Recognized options:
 
 =over
@@ -145,6 +147,7 @@ sub _search_packages {
 	my @found;
 	$qv = '(?i)' . $qv if $options{caseinsensitive};
 
+	# First: try to find an exact match
 	if (!$options{fuzzy}) {
 	    #- try to search through provides.
 	    my @l = map {
@@ -164,6 +167,7 @@ sub _search_packages {
 	    _findindeps($urpm, \%found, $qv, $v, $options{caseinsensitive}, $options{src});
 	}
 
+	# Second pass: try to find a partial match (substring) [slow]
 	foreach my $id (build_listid_($urpm)) {
 	    my $pkg = $urpm->{depslist}[$id];
 	    ($options{src} ? $pkg->arch eq 'src' : $pkg->is_arch_compat) or next;
@@ -174,14 +178,12 @@ sub _search_packages {
 	    if (!$options{fuzzy}) {
 		if ($pack eq $v) {
 		    $exact{$v} = $id;
-		    next;
 		} elsif ($pack_a eq $v) {
 		    push @{$exact_a{$v}}, $id;
-		    next;
 		} elsif ($pack_ra eq $v || $options{src} && $pack_name eq $v) {
 		    push @{$exact_ra{$v}}, $id;
-		    next;
 		}
+		next;
 	    }
 	    if ($pack =~ /$qv/) {
 		next if member($pack, @found);
@@ -218,7 +220,7 @@ sub _search_packages {
 		$urpm->{error}(N("No package named %s", $v));
 		values(%l) != 0 and $urpm->{error}(
 		    N("The following packages contain %s: %s",
-			$v, urpm::util::formatList(4, sort { $a cmp $b } keys %l)) . "\n" . 
+			$v, formatList(4, sort { $a cmp $b } keys %l)) . "\n" . 
 		    N("You should use \"-a\" to use all of them")
 		);
 		$result = 0;
@@ -351,6 +353,7 @@ sub resolve_dependencies {
 		$need_restart = _resolve_priority_upgrades($urpm, $db, $state, $state->{selected}, \@l, %options);
 	    }
 	}
+	$urpm->{options}{'split-length'} = 0 if $need_restart;
     }
     $need_restart;
 }
@@ -399,9 +402,9 @@ sub _resolve_priority_upgrades {
     my %priority_requested = map { $_->id => undef } @$priority_pkgs;
 
     $urpm->resolve_requested($db, \%priority_state, \%priority_requested, %options);
-    if (grep { ! exists $priority_state{selected}{$_} } keys %priority_requested) {
+    if (any { ! exists $priority_state{selected}{$_} } keys %priority_requested) {
 	#- some packages which were selected previously have not been selected, strange!
-    } elsif (grep { ! exists $priority_state{selected}{$_} } keys %$selected) {
+    } elsif (any { ! exists $priority_state{selected}{$_} } keys %$selected) {
 	#- there are other packages to install after this priority transaction.
 	%$state = %priority_state;
 	$need_restart = 1;
@@ -489,9 +492,12 @@ sub find_packages_to_remove {
 			push @m, scalar $p->fullname;
 			$found = 1;
 		    });
-		$found and next;
 
-		push @notfound, $_;
+		if ($found) {
+		    next;
+		} else {
+		    push @notfound, $_;
+		}
 	    }
 	    if (!$options{force} && @notfound && @$l > 1) {
 		$options{callback_notfound} && $options{callback_notfound}->($urpm, @notfound)
@@ -565,7 +571,7 @@ sub _prohibit_packages_that_would_be_removed {
     });
 
     grep {
-	! grep { $base{$_} } rejected_unsatisfied($state, $_);
+	! any { $base{$_} } rejected_unsatisfied($state, $_);
     } intersection(\@to_remove, \@base_fn);
 }
 
@@ -688,7 +694,7 @@ sub translate_why_removed_one {
     my $closure = rejected_closure($state, $fullname) or return $fullname;
 
     my ($from) = keys %$closure;
-    my ($whyk) = keys %{$closure->{$from}};
+    my ($whyk) = sort { $b ne 'avoid' } keys %{$closure->{$from}};
     my $whyv = $closure->{$from}{$whyk};
     my $frompkg = $urpm->search($from, strict_fullname => 1);
     my $s = do {
@@ -713,7 +719,7 @@ sub translate_why_removed_one {
 }
 
 sub _libdb_version { $_[0] =~ /libdb-(\S+)\.so/ ? version->new("v$1") : () }
-sub _rpm_version() { `rpm --version` =~ /version ([0-9.]+)$/ ? version->new("v$1") : () }
+sub _rpm_version() { `rpm --version` =~ /version ([0-9.]+)(?:-beta.*)?$/ ? version->new("v$1") : () }
 
 sub should_we_migrate_back_rpmdb_db_version {
     my ($urpm, $state) = @_;
